1000	%title "rdb_demo2_dynamic_xxx.bas"
	%ident "version 100.1"
	!
	!========================================================================================================================
	! title  : rdb_demo2_dynamic_xxx.bas
	! author : Neil Rieck	(http://www3.sympatico.ca/n.rieck)
	! purpose: to demo RDB71 concepts on ICSIS
	! notes  : also see file rdb_demo2_dynamic_xxx_bas.sqlmod (which must be complied using SQLMOD71)
	! history:
	! ver who when   what
	! --- --- ------ --------------------------------------------------------------------------------------------------------
	! 100 NSR 040907 1. original code
	!========================================================================================================================
	! DCL Build Instructions:
	!	DCL command						result
	!	basic	rdb_demo2_dynamic_xxx.bas			rdb_demo2_dynamic_xxx.obj
	!	sql$mod	rdb_demo2_dynamic_bas_xxx.sqlmod		rdb_demo2_dynamic_bas_xxx.obj
	!	link	rdb_demo2_dynamic_xxx.obj		, -
	!		rdb_demo2_dynamic_bas_xxx.obj		, -
	!		SQL$USER/library				rdb_demo2_dynamic_xxx.exe
	!
	! note: "SQL$USER" is a special logical created by executing "@sys$library:RDBVMS_SETVER.COM reset" or
	!	"@csmis$com:RDB_INIT_ENVIRONMENT.COM". This logical is currently defined as "SYS$COMMON:[SYSLIB]SQL$USER71.OLB"
	!	which could change with different versions of RDB (many versions can co-exist on the same system) so never
	!	hard code the library specification. Always use logical "SQL$USER".
	!========================================================================================================================
	option type=explicit
	!
	!	optional constant declarations (placed here to show that they exist)
	!
	%include "sys$library:SQL_LITERALS.BAS"					! end-of-stream etc. in SQLCODE
	%include "sys$library:COSIMSG.BAS"					!
	%include "sys$library:RDBMSGS.BAS"					!
	%include "sys$library:RDMSMSG.BAS"					!
	%include "sys$library:SQLSTATE.BAS"					! required with SQLSTATE
	%include "sys$library:SQL_MSG.BAS"					! 
	%include "sys$library:SQL_SQLCA.BAS"					! Sql Communications Area (SCA)
	%include "RDB_SQLDA.BAS"						! Sql Descriptor Area
	!
	!	older SQLMOD routines used SQLCODE but Rdb documentation (Oracle Rdb7 "Guide to SQL Programming" 10.2.1) states
	!	that new applications should use SQLSTATE (but you can also use both)
	!
    %let %new_method=1%								! use SQLCODE
    %if  %new_method=0% %then							! OLD
        !
        !       SQLCODE values: 0       = succes
        !                       100     = end of data
        !                       < 0     = some kind of error
	!
	declare long sql_code							! from SQLCODE (a long integer)
    %else									! NEW
	!				++-------- class code
	!				||+++----- sub-class code
	!	some SQLSTATE values:	00000 success
	!				01000 warning
	!				02000 no more data
	!				2200F zero-length string
	!				0H000 Invalid SQL state
	!				S7000 bad version
	!
	map(sqlstate)	string sql_state	= 5				! from SQLSTATE (a 5 char string)
	map(sqlstate)	string sql_class	= 2	,&
			string sql_subclass	= 3
    %end %if
	!
	declare	string	sql_stmt$, junk$,					!						&
		long	type0%,  type1%, type2%,				!						&
			i%, j%, test_case%, junk%				!
	!
	record f255								!
	    string	f255 = 255						!
	end record f255
	!
	!========================================================================================================================
	!	main
	!========================================================================================================================
	main:
	!
	!	init SQLDA
	!
	SQLDA::SQLDAID		= "SQLDA   "
	SQLDA::SQLN		= k_max_SQLVAR%					! see file: "RDB_SQLDA.BAS"
	!
	!	init other stuff
	!
	test_case% = 0%
	!------------------------------------------------------------------------------------------------------------------------
	!	test case entry point 
	!------------------------------------------------------------------------------------------------------------------------
	loop:
	select test_case%
	    case 0
		sql_stmt$ = "SELECT NAME, TEL1, ROWID FROM CUSTOMER"			! an SQL command for our cursor
	    case 1
		sql_stmt$ = "SELECT NAME, TEL1, ROWID FROM CUSTOMER WHERE CITY = 'Kitchener'"
	    case 2
		sql_stmt$ = "SELECT NAME, TEL1, POSTAL, ROWID FROM CUSTOMER ORDER BY POSTAL"
	    case 3									! I wonder what type ROWID will be?
		sql_stmt$ = "SELECT NAME, TEL1, ROWID FROM CUSTOMER WHERE CITY = 'Barrie'"
	    case else
		goto fini							!
	end select
	print "=========================================="
	print "-i- starting test case: "+ str$(test_case%)
	print "-i- sql_stmt$ = "; sql_stmt$
	!
	!	process the SQL command line
	!
	call PREP_STMT(sql_state, sql_stmt$)					!
	print "-i- prep_stmt sql_state: "; sql_state				!
	!
	!	now describe it to the SQLDA
	!
	call DESCRIBE(sql_state, SQLDA)						! now set up the SQL Dynamic Area
	print "-i- describe sql_state: "; sql_state				!
	!
	!	<<< set up storage locations for the parameters registered in the SQLDA >>>
	!
	print "-i- setting up storage locations for the SQLDA"
!	SQLDA::SQLVAR(0)::SQLDATA	= loc( param(0)::f255 )			x
!	SQLDA::SQLVAR(1)::SQLDATA	= loc( param(1)::f255 )			x
!	SQLDA::SQLVAR(2)::SQLDATA	= loc( param(2)::f255 )			x
	!
	!	declare enough variables to run this demo (not all will be used)
	!
	i% = SQLDA::SQLD							! how many parameters are required?
	print "-i- allocating memory"
	dim f255		param  (i%)					! need some fixed length strings
	dim string		param$ (i%)					!
	dim byte		param_b(i%)					! need some bytes
	dim word		param_w(i%)					! need some words
	dim long		param_l(i%)					! need some longs
	dim basic$quadword	param_q(i%)					! need some longs
	!
	for i% = 0 to (SQLDA::SQLD - 1)
	    !
	    !	see "Oracle Rdb ver 7.1.2 - SQL Reference Manual" - Volume 5 - Page D-13
	    !	or file "sys$library:SQL_LITERALS.BAS" (then search for SQLDA.SQLVAR.SQLTYPE)
	    !
	    select SQLDA::SQLVAR(i%)::SQLTYPE
		case 453, 449							! SQLCHAR, SQLDA_VARCHAR
		    SQLDA::SQLVAR(i%)::SQLDATA	= loc( param(i%)::f255 )	!	we need a string
		case 497							! INTEGER
		    SQLDA::SQLVAR(i%)::SQLDATA	= loc( param_l(i%))		!	we need a long
		case 501							! SMALLINT
		    SQLDA::SQLVAR(i%)::SQLDATA	= loc( param_w(i%))		!	we need a word
		case 505							! BIGINT
		    SQLDA::SQLVAR(i%)::SQLDATA	= loc( param_q(i%))		!	we need a quadword
		case 515							! TINYINT
		    SQLDA::SQLVAR(i%)::SQLDATA	= loc( param_b(i%))		!	we need a byte
		case else							! oops...
		    SQLDA::SQLVAR(i%)::SQLDATA   = loc( param(i%)::f255 )	!	default to something big
		    print "-w-parameter: "+ str$(i%)
		    print "-w-type     : "+ str$( SQLDA::SQLVAR(i%)::SQLTYPE )
		    print "-w-len      : "+ str$( SQLDA::SQLVAR(i%)::SQLLEN )
		    print "-w-this program doesn't support SQLTYPE: "; str$( SQLDA::SQLVAR(i%)::SQLTYPE )
	    end select
	next i%
	!
	!	now open the cursor
	!
	call OPEN_CURSOR9(sql_state)						! now open the cursor
	print "-i- CURSOR OPEN sql_state: "; sql_state				!
	!
	!	now use the cursor to extract the data
	!
	i% = 0
	mat param$ = nul$							! zap dynamic array
	until sql_state <> "00000"						!
	    call fetch_cursor9(sql_state, SQLDA)				! read a row of data
	    print "-i- CURSOR FETCH sql_state: "; sql_state			!
										!
	    if (sql_state = "00000") then					! if no error, then process
		print "-i-parameter count: "+ str$(SQLDA::SQLD)			!
		for i% = 0 to (SQLDA::SQLD -1)					!
		    print "-i-parameter: "+ str$(i%)				!
		    select SQLDA::SQLVAR(i%)::SQLTYPE				!
			case 453, 449						! CHAR, VARCHAR
			    param$(i%) = left$(param(i%)::f255, SQLDA::SQLVAR(0)::SQLLEN)
			    print "-i-len      : "+ str$( SQLDA::SQLVAR(i%)::SQLLEN )
			    print "-i-data     : "; param$(i%);
			    select SQLDA::SQLVAR(i%)::SQLTYPE
				case 453
				    print " (CHAR)"
				case else
				    print " (VARCHAR)"
			    end select
			    if SQLDA::SQLVAR(i%)::SQLLEN = 8 then		! if it looks like ROWID
				print "-i-peeking at characters in possible ROWID (quick hack)"
				for j% = 1 to 8					! then peek
				    print "-i-char pos: "+str$(j%)+" ascii: "; asc( mid$(param$(i%),j%,1) )
				next j%						!
				print "-i-type     : "+ str$( SQLDA::SQLVAR(i%)::SQLTYPE )
			    end if						!
			case 497						! Integer
			    print "-i-len      : "+ str$( SQLDA::SQLVAR(i%)::SQLLEN )
			    print "-i-data     : "; param_l(i%); " (32-bit)"	!	use the long
			case 501						! SMALLINT
			    print "-i-len      : "+ str$( SQLDA::SQLVAR(i%)::SQLLEN )
			    print "-i-data     : "; param_w(i%); " (16-bit)"	!	use the word
			case 505						! BIGINT
			    print "-i-len      : "+ str$( SQLDA::SQLVAR(i%)::SQLLEN )
		!~~~	    print "-i-data     : "; param_q(i%) 		x	use the quad
			    print "-i-data     : (unprintable 64 bit number)"	!
			case 515						! TINYINT
			    print "-i-len      : "+ str$( SQLDA::SQLVAR(i%)::SQLLEN )
			    print "-i-data     : "; param_b(i%); " (byte)"	!	use the byte
			case else						!
			    print "-w-parameter: "+ str$(i%)
			    print "-w-type     : "+ str$( SQLDA::SQLVAR(i%)::SQLTYPE )
			    print "-w-len      : "+ str$( SQLDA::SQLVAR(i%)::SQLLEN )
			    print "-w-this program doesn't support SQLTYPE: "; str$( SQLDA::SQLVAR(i%)::SQLTYPE )
		    end select
		next i%
	    else								!
		print "-e- sql_state: "; sql_state				!
	    end if								!
	    i% = i% + 1								! advance our parameter counter
	next									!
	!
	!	closing the cursor reduces overhead
	!
	call close_cursor9(sql_state)
	print "-i- CURSOR CLOSE sql_state: "; sql_state
	!
	!	run the next test case
	!
	print "-i-end of test case: "+ str$(test_case%)				!
	test_case% = test_case% + 1						! advance to the next test case
	goto loop								!	***--->>>
	!========================================================================================================================
	!	That's all she wrote. Adios...
	!========================================================================================================================
	fini:
	end
